BubbleLoot = LibStub("AceAddon-3.0"):NewAddon("BubbleLoot", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")

-- OPTIONS
-- enableBL - toggles handling loot (ie. it ignores LOOT_OPENED events)
-- enableML - toggles handling the actual loot assignment (ie. just displays winner)
-- enableRB - toggles parsing of raid bids (ie. "bid", "off", "pvp")
-- enableWB - toggles parsing of whisper bids (ie. ignores whisper "bid", "off", "pvp")
-- enableSB - toggles echo of loot bid receipt into the raid channel
-- enableBS - toggles printing of summary of all bids or just winner
-- enableGD - toggles use of guild DKP to rank bids
-- bidTime - input value for length of bid
-- defaultSharder - input value for who to send shards to
local options = {
	name = "BubbleLoot",
	handler = BubbleLoot,
	type = "group",
	args = {
		enableBL = {
			order=1, type = "toggle",
			name = "Enable Bubble Loot",
			desc = "Enables the master looting management by BubbleLoot",
			get = function(info) return(BubbleLoot.db.profile.enableBL) end,
			set = function(info, key) BubbleLoot.db.profile.enableBL=key end
		},enableML = {
			order=2, type = "toggle",
			name = "Master Loot Assignment",
			desc = "Enables the actual loot assignment by BubbleLoot",
			get = function(info) return(BubbleLoot.db.profile.enableML) end,
			set = function(info, key) BubbleLoot.db.profile.enableML=key end
		},enableRB = {
			order=3, type = "toggle",
			name = "Raid Bids",
			desc = "Enables people to perform bids via raid",
			get = function(info) return(BubbleLoot.db.profile.enableRB) end,
			set = function(info, key) BubbleLoot.db.profile.enableRB=key end
		},enableWB = {
			order=4, type = "toggle",
			name = "Whisper Bids",
			desc = "Enables people to perform bids via whisper",
			get = function(info) return(BubbleLoot.db.profile.enableWB) end,
			set = function(info, key) BubbleLoot.db.profile.enableWB=key end
		},enableSB = {
			order=5, type = "toggle",
			name = "Enable Silent Bids",
			desc = "Does not echo bids to raid channel received via whisper",
			get = function(info) return(BubbleLoot.db.profile.enableSB) end,
			set = function(info, key) BubbleLoot.db.profile.enableSB=key end
		},enableGD = {
			order=6, type = "toggle",
			name = "Enable Guild DKP",
			desc = "Uses guild DKP rather than items that night as the discriminator",
			get = function(info) return(BubbleLoot.db.profile.enableGD) end,
			set = function(info, key) BubbleLoot.db.profile.enableGD=key end
		},enableBS = {
			order=7, type = "toggle",
			name = "Bid Summary",
			desc = "Posts bid summary as well as the winner to RW",
			get = function(info) return(BubbleLoot.db.profile.enableBS) end,
			set = function(info, key) BubbleLoot.db.profile.enableBS=key end
		},enableAQ = {
			order=7, type = "toggle",
			name = "Enable Az Quotes",
			desc = "Whispers random azroar loot whore comments to greedy bidders",
			get = function(info) return(BubbleLoot.db.profile.enableAQ) end,
			set = function(info, key) BubbleLoot.db.profile.enableAQ=key end
		},bidTime={
			order=9, type = "input",
			name = "BidTime",
			desc = "How long people have to bid for each loot item",
			get = function(info) return(BubbleLoot.db.profile.bidTime) end,
			set = function(info, key) BubbleLoot.db.profile.bidTime=key end
		},defaultSharder = {
			order=10, type = "input",
			name = "Default Sharder",
			desc = "Default person to master loot shards to",
			get = function(info) return(BubbleLoot.db.profile.defaultSharder) end,
			set = function(info, key) BubbleLoot.db.profile.defaultSharder=key end
		}
	}
}
local defaultOptions = {
	profile = {
		enableBL = false,
		enableML = false,
		enableRB = true,
		enableWB = true,
		enableSB = false,
		enableGD = false,
		enableBS = true,
		bidTime = 15,
		defaultSharder = nil
	}
}

-- REGISTER_OPTIONS
-- Initialises the BubbleLootDB variable and registers the base options
function BubbleLoot:RegisterOptions()
	BubbleLoot.db = LibStub("AceDB-3.0"):New("BubbleLootDB", defaultOptions, "Default")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("BubbleLoot", options)
	BubbleLoot.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("BubbleLoot", "BubbleLoot")
end
