-- OVERVIEW
-- TBC

-- ON_INITIALIZE
-- This handler does the initial loading of all dependent libraries and sets up the config
-- GUI.  It also provides an empty entry for the loot list if it doesn't exist yet.
function BubbleLoot:OnInitialize()
	self:RegisterOptions()
	self:RegisterVariables()
	self:RegisterChatCommand("bl", "ChatCommand")
	self:RegisterChatCommand("bubbleloot", "ChatCommand")
	self:Print("BubbleLoot 0.1 Initialised")
	self.data.guildLib=LibStub("LibGuild-1.0")
	if (BubbleLootList == nil) then
		BubbleLootList = {}
	end
end

-- ON_INITIALIZE
-- Just register our event handlers below to the events we're interested in listening for
function BubbleLoot:OnEnable()
	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("CHAT_MSG_RAID")
	self:RegisterEvent("CHAT_MSG_RAID_LEADER")
	self:RegisterEvent("CHAT_MSG_PARTY")
	self:RegisterEvent("LOOT_OPENED")
	self:RegisterEvent("LOOT_CLOSED")
	self:RegisterEvent("PARTY_LOOT_METHOD_CHANGED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:UpdateLootMethod()
	self:UpdateGroupTable()
end
function BubbleLoot:OnDisable()
end

-- EVENT HANDLERS
-- Subs that invoke real handlers just to make the code a little cleaner
function BubbleLoot:CHAT_MSG_LOOT(event, msg, user) self:HandleLootAssign(msg, user)end
function BubbleLoot:CHAT_MSG_RAID(event, msg, user) self:HandleCmd(msg, user, false)end
function BubbleLoot:CHAT_MSG_RAID_LEADER(event, msg, user) self:HandleCmd(msg, user, false)end
function BubbleLoot:CHAT_MSG_PARTY(event, msg, user) self:HandleCmd(msg, user, false)end
function BubbleLoot:CHAT_MSG_WHISPER(event, msg, user) self:HandleCmd(msg, user, true) end
function BubbleLoot:PARTY_LOOT_METHOD_CHANGED() self:UpdateLootMethod() end
function BubbleLoot:PARTY_MEMBERS_CHANGED() self:UpdateGroupTable() end
function BubbleLoot:RAID_ROSTER_UPDATE() self:UpdateGroupTable() end

function BubbleLoot:LOOT_OPENED()
	if(self:ShouldWeProcessLoot()) then 
		self.data.bDoingLoot=false
		self.data.iCurrentSlot=1
		self:ProcessLoot()
	end
end
function BubbleLoot:LOOT_CLOSED()
	if(self.data.bDoingLoot) then
		SendChatMessage("<BL> Automated looting cancelled", "RAID_WARNING")
	end
	self:CancelTimer(timerBid)
	self.data.bDoingLoot=false
end

-- CHAT COMMAND HANDLER
-- Handles all the commandline arguments for our addon including:
--   LOOT - lists all loot for a player
--   LOOT <player> - lists all loot for specified player
--   RESET - resets the loot table (automatic reset disabled in alpha)
--   GUILD <player> - returns the DKP for a specified player
function BubbleLoot:ChatCommand(input)
	if ((not input) or (input:trim() == "")) then
		InterfaceOptionsFrame_OpenToCategory(BubbleLoot.optionsFrame)
	elseif (input:lower() == "loot") then
		self:ListLoot(nil)
	elseif(string.match(input:lower(), "loot%s+")) then
		local filter=string.match(input, "%a+%s+(%a+)")
		self:ListLoot(nil, filter)
	elseif (input:lower() == "reset") then
		self:Print("<BL> Loot reset")
		BubbleLootList = {}
		self.data.tableTally={}
	elseif (input:lower() == "forum") then
		self:DoForum()
	elseif(input:lower() == "dkp") then
		self:Print("<BL> Your DKP is currently: " .. self:GetDKP(UnitName("player")))
	elseif((string.sub(input:lower(),1, 3)=="dkp") and string.match(input:lower(), "dkp%s+")) then
		local filter=string.match(input, "%a+%s+(%a+)")
		self:Print("<BL> DKP for user " .. filter .. " is currently: " .. self:GetDKP(filter))
	elseif(input:lower() == "test") then
		self:Print("testing...")
		self:RegisterBid('Bubblecannon', 3, 50)
		self:RegisterBid('Schmeedle', 3, 17)
		self:RegisterBid('Wolfnite', 2, 71)
		self:RegisterBid('Azroar', 1, 99)
		self:DoBidSummary()
	elseif(input:lower()=="resetbids") then
		self:ResetBids()
	elseif(string.sub(input:lower(),1,3) == "bid") then
		local user, bid, roll=string.match(input, "%a+%s+(%a+)%s+(%a+)%s+(%d+)")
		self:Print("User: " .. user .. " Bid: " .. bid .. " Roll:" .. roll)
		self:RegisterBid(user, mapBid2Ndx[bid], tonumber(roll))
	elseif(input:lower() == "eval") then
	
	elseif(input:lower() == "azsays") then
		self:DoAzQuote("Bubblecannon")
	elseif (string.match(input:lower(), "(%a+)%s+")=="guild") then
		local name=string.match(input, "%a+%s+(%a+)")
		self:Print("DKP for " .. name .. " is " .. dkp)
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(BubbleLoot, "bl", "BubbleLoot", input)
	end
end


-- HANDLE_CMD
-- This function handles the registration of commands from whispers.  If the whisper command
-- matches on the of the bid types, it will record the bid if bidding is open.  It will also
-- handle loot queries and provides a very rudimentary help
-- TODO: improve the help in some manner
function BubbleLoot:HandleCmd(msg, user, isWhisper)
	local roll = random(1, 100)
--	need to do a check to see if im in a raid or a party first
--	if((msg:lower()=="bid") or (msg:lower()=="off") or (msg:lower()=="pvp")) then
	local iBidNdx=self.data.mapBid2Ndx[msg:lower()]
	if(iBidNdx~=nil) then
		if(not self.data.bAmML) then
			if(isWhisper) then
				SendChatMessage("<BL> Not master looter.  Bid ignored", "WHISPER", nil, user)
			end
			return
		elseif((isWhisper and self.db.profile.enableWB) or (not isWhisper and self.db.profile.enableRB))then
			if(self.data.tableGroup[user]==nil) then
				SendChatMessage("<BL> Not in party/raid group.  Bid ignored", "WHISPER", nil, user)
				return
			elseif(self.data.bDoingLoot == false) then
				SendChatMessage("<BL> Bids closed", "WHISPER", nil, user)
			else
				self:RegisterBid(user, iBidNdx, random(1, 100))
			end
		end
	elseif((msg:lower() == "wait") and self.data.bDoingLoot) then
		self:Print("got wait")
		self:HandleWait()
	elseif((msg:lower() == "azsays") and isWhisper)then
		self:DoAzQuote(user)
	elseif(msg:lower() == "loot") then
		SendChatMessage("<BL> Your loot recorded in this raid is as follows:", "WHISPER", nil, user)
		self:ListLoot(user, user)
	elseif((string.sub(msg:lower(),1, 4)=="loot") and string.match(msg:lower(), "loot%s+")) then
		local filter=string.match(msg, "%a+%s+(%a+)")
		SendChatMessage("<BL> Loot for user " .. filter .. " recorded in this raid is as follows:", "WHISPER", nil, user)
		self:ListLoot(user, filter)
	elseif(msg:lower() == "dkp") then
		SendChatMessage("<BL> Your DKP is currently: " .. self:GetDKP(user), "WHISPER", nil, user)
	elseif((string.sub(msg:lower(),1, 3)=="dkp") and string.match(msg:lower(), "dkp%s+")) then
		local filter=string.match(msg, "%a+%s+(%a+)")
		SendChatMessage("<BL> DKP for user " .. filter .. " is currently: " .. self:GetDKP(filter), "WHISPER", nil, user)
	elseif(msg:lower() == "help") then
		SendChatMessage("<BL> BubbleLoot only accepts BID, OFF, PVP, LOOT and DKP commands", "WHISPER", nil, user)
		SendChatMessage("<BL> Whisper this user with either BID, OFF, or PVP to register a bid on an item when prompted", "WHISPER", nil, user)
		SendChatMessage("<BL> Or if the MasterLooter has set it up, you can bid in raid chat", "WHISPER", nil, user)
		SendChatMessage("<BL> You can see your own loot by whispering me 'loot' or someone elses by 'loot name'", "WHISPER", nil, user)
		SendChatMessage("<BL> You can see your own dkp by whispering me 'dkp' or someone elses by 'dkp name'", "WHISPER", nil, user)
		SendChatMessage("<BL> Note: you must use the correct exact case for the players name (eg. Bubblecannon not bubblecannon)", "WHISPER", nil, user)
	end
end


-- GET_DKP
-- This function is simply a cleanliness function to extract the DKP from the guild notes
-- that adheres to the format: Net:<digits> Total:<digits>
function BubbleLoot:GetDKP(name)
	local onote=self.data.guildLib:GetOfficerNote(name)
	if(onote==nil) then
		return(0)
	else
		local dkp=string.match(onote, "Net:(%-?%d+)")
		if(dkp==nil) then
			return(0)
		else
			return(tonumber(dkp))
		end
	end
end

-- DO_AZ_QUOTE
-- This function performs the code to obtain a random AzLootWhoreQuote
function BubbleLoot:DoAzQuote(user)
	local iNdx=random(1, #self.data.tableAzQuotes);
	SendChatMessage("<BL> AzSays: " .. self.data.tableAzQuotes[iNdx] , "WHISPER", nil, user)
end

-- HANDLE_WAIT
-- This cancels the current timer and restarts it
function BubbleLoot:HandleWait()
	local iNewBidTime=tonumber(BubbleLoot.db.profile.bidTime)*2
	self:CancelTimer(self.data.timerBid)
	SendChatMessage("<BL> Extending timer. You now have " .. tostring(iNewBidTime) .. " secs left.", "RAID_WARNING")
	self.data.timerBid=self:ScheduleTimer("HandleTimer", iNewBidTime)
end

function BubbleLoot:ResetBids()
	tableBids={}
end

-- UPDATE_GROUP_TABLE
-- This function is invoked at initialisation and when the party/raid changes in some manner.  It
-- determines if its part of a raid or party and updates accordingly
-- TODO: check if our name shows up in the raid list
function BubbleLoot:UpdateGroupTable()
	self.data.tableGroup={}
	if(GetNumRaidMembers()>0) then
		self.data.bInRaid=true
		for i=1, GetNumRaidMembers() do
			local name=GetRaidRosterInfo(i)
			if(name==nil) then
				self:Print("Raid index " .. i .. " is nil")
			else
				self.data.tableGroup[name]=i
			end
		end
	else
		self.data.bInRaid=true
		for i=1, GetNumPartyMembers() do
			local name=UnitName("party" .. i)
			self.data.tableGroup[name]=i
		end
	self.data.tableGroup[UnitName("player")]= (GetNumPartyMembers()+1)
	end
end
