
-- SHOULD_WE_PROCESS_LOOT
-- This function checks to see if:
-- (a.) the addon has been enabled
-- (b.) the current loot method is master
-- (c.) that we are the assigned loot master
-- (d.) lastly, we check to see if there are any items worth rolling on in the loot
function BubbleLoot:ShouldWeProcessLoot()
	if((self.db.profile.enableBL) and (self.data.bAmML))then
		for i=1, GetNumLootItems() do
			if(self:IsRollableItem(GetLootSlotLink(i))) then
				return(true)
			end
		end
	end
	return (false)
end

-- IS_ROLLABLE_ITEM
-- This function takes an item link and return true or false if it's worth rolling on.  The
-- threshold is rare items for a party and epic items for a raid.  It also uses a blacklist
-- filter to avoid logging emblems/gems/etc.
function BubbleLoot:IsRollableItem(item)
	if(item==nil) then return(false) end
	local itemId = string.match(item, "item:(%d+)")
	local itemName, itemLink, itemQual = GetItemInfo(itemId)
	if((self.data.bInRaid and itemQual==4) or (itemQual==3)) then
		for i,v in ipairs(self.data.mapFilter) do
			if(itemName == v) then
				return(false)
			end
		end
		return(true)
	end
	return false
end

-- PROCESS_LOOT
-- This iterates through the items in an open loot window and automates the bidding process
-- on the master looters behalf
function BubbleLoot:ProcessLoot()
	for i=self.data.iCurrentSlot, GetNumLootItems() do
		self.data.iCurrentSlot=i
		local itemLink=GetLootSlotLink(i)
		if(self:IsRollableItem(itemLink)) then
			self.data.strItem=itemLink
			self.data.bDoingLoot=true
			self.data.iCurrentSlot=i+1
			self.data.tableBids={}
			SendChatMessage("<BL> Post bids for " .. itemLink .. " " .. self.db.profile.bidTime .. " secs left", "RAID_WARNING")
			self.data.timerBid=self:ScheduleTimer("HandleTimer", tonumber(self.db.profile.bidTime))
			break;
		end
	end
end

-- HANDLE TIMER
-- This function handles the timer scheduled by the ProcessLoot function.  It iterates through
-- the receved bids and yadda yadda
-- tableBids is keyed by index with tuple(username, int bid, int roll)
function BubbleLoot:HandleTimer()
	self:CancelTimer(timerBid)
	if(not self.data.bDoingLoot) then return end
	self.data.bDoingLoot=false
	
	if(#self.data.tableBids==0) then
		SendChatMessage("<BL> No bids received.  Item being sharded by defaultSharder", "RAID_WARNING")
	else
		local winUser, winBidNdx, winRoll=self:DecideBidWinner() 
		local winDkp=0
		local winTally=0
		if(self.db.profile.enableGD) then
			winDkp=self:GetDKP(winUser)
		else
			winTally=self.data.tableTally[winUser]
		end
		if(self.db.profile.enableBS) then
			self:DoBidSummary()
		end
		SendChatMessage("<BL> " .. winUser .. " won with " .. self.data.mapNdx2Bid[winBidNdx] .." roll of " .. winRoll, "RAID_WARNING")
		SendChatMessage("<BL> Gratz on the win of " .. self.data.strItem, "WHISPER", nil, winUser)
	end
--	GiveMasterLoot(i, 1)
	self:ProcessLoot()
end

-- DECIDE_BID_WINNER
function BubbleLoot:DecideBidWinner()
--	now we need to evaluate the bids
--	now we enumerate through the bids working out who will win
--	pvp is 0, off is 1, main is 2
	local bidCount=0
	local winType=0
	local winUser=nil
	local winRoll=0
	local winDkp=0
	local winTally=0
	for i,v in ipairs(self.data.tableBids) do
		local winFlag=false
		bidCount=bidCount+1
		self:Print("Found bid from " .. v[1] .. " of type " .. self.data.mapNdx2Bid[v[2]] .. " and roll " .. tostring(v[3]))
--		if currently no winner, or a higher bid type winner then the first bid becomes the potential winner
		if((winUser==nil) or (v[2]>winType))then
			winFlag=true;
--		otherwise if the bid is the same type as the current winner, do a /roll or DKP compare
		elseif(v[2]==winType) then
			if((v[2]==3) and self.db.profile.enabledGD) then
				local dkp=self:GetDKP(v[1])
				if(dkp>winDkp) then
					winFlag=true
				elseif((dkp==winDkp) and (v[3]>winRoll)) then
					winFlag=true
				end
			else
				local tally=self.data.tableTally[v[1]]
				if(tally~=nil) then
					tally=tally[1] -- this is so dumb
				end
				if((winTally==nil) and (tally==nil)) then
					winFlag= (v[3]>winRoll)
				elseif(tally==nil) then
					winFlag=true
				elseif(winTally==nil) then
					winFlag=false
				elseif(tally<winTally) then
					winFlag=true
				elseif((tally==winTally) and (v[3]>winRoll)) then
					winFlag=true
				end
			end
		end
--		if this bids wins, then set the win running information
		if(winFlag) then
			winUser=v[1]
			winType=v[2]
			winRoll=v[3]
			winDkp=self:GetDKP(v[1])
			winTally=self.data.tableTally[v[1]]
		end
	end
	return winUser, winType, winRoll
end

-- DO_LOOT_SUMMARY
-- This parses the bids and constructs multiple output strings
function BubbleLoot:DoBidSummary()
	self:Print("Doing bid summary")
--	TODO: fix this local to be more generic
	local bidStr={nil, nil, nil}
	for i,v in ipairs(self.data.tableBids) do
--		construct the summary as USER (ROLL, DISCRIM)
		local bid=v[1] .. "(" .. v[3] .. ","
		if ((self.db.profile.enableGD) and (i==3))then
			self:Print(v[1] .. " " .. self:GetDKP(v[1]))
			bid = bid .. self:GetDKP(v[1]) .. "D)"
		else
			local tally=self.data.tableTally[v[1]]
			if(tally~=nil) then
				tally=tally[1]
			else
				tally=0
			end
			bid = bid .. tostring(tally) .. "T)"
		end
--		if this is th first string, then just assign it.  Otherwise append the string
		if(bidStr[v[2]]==nil) then
			bidStr[v[2]]=bid
		else
			bidStr[v[2]]=bidStr[v[2]] .. ", " .. bid
		end
	end
--	now iterate through the array and summarise any bids to raid warning
	for i=1, #bidStr do
		if(bidStr[i]) then
			SendChatMessage("<BL> " .. self.data.mapNdx2Bid[i] .. ": " .. bidStr[i], "RAID_WARNING")
		end
	end
end

-- DO_FORUM
-- This function is a very dodgy utility function used to generate a simple PHPBB code
-- output.  However, it needs to be altered to be a lot more functional.  Generating a single
-- string and placing that inside a large frame would be far more useful.
-- TODO: fix this to be far more useful than it currently is including better formatting and GUI
function BubbleLoot:DoForum()
	for i,v in ipairs(BubbleLootList) do
		local itemName = GetItemInfo(v[3])
		self:Print("[wowitem]" .. itemName .."[/wowitem] " .. v[1])
	end
end


-- REGISTER_BID
-- This function performs the code to register a bid in the data store.  It is seperated from HandleCmd()
-- to provide modularity and allow a test harness
-- Note: This method does not do sanity checking on bDoingLoot
function BubbleLoot:RegisterBid(user, bidNdx, roll)
	local strBid=self.data.mapNdx2Bid[bidNdx]
--	SendChatMessage("<BL> " .. strBid .. " registered for " .. user .. " with roll " .. roll, "WHISPER", nil, user)
--	since we've had to change the tableBids to allow iteration, we now need to check for duplicates
	local bMatch=false
	for i, v in ipairs(self.data.tableBids) do
		if(v[1]==user) then
			v[2]=bidNdx
			roll=v[3]
			bMatch=true
		end
	end
	if(not bMatch) then
		table.insert(self.data.tableBids, {user, bidNdx, roll})
	end
	if(not self.db.profile.enableSB) then
		if(self.data.bInRaid) then
			SendChatMessage("<BL> " .. strBid .. " registered for " .. user .. " with roll " .. roll, "RAID")
		else
			SendChatMessage("<BL> " .. strBid .. " registered for " .. user .. " with roll " .. roll, "PARTY")
		end
	end
--	now do the azlootwhore check
	if(self.data.tableTally[user]~=nil) then
		BubbleLoot:DoAzQuote(user)
	end
end

-- LIST_LOOT
-- This function simply lists all the loot for a particular user
-- [BubbleLootList is keyed by index with tuple(str user, str item_link, int item_id, int bid)]
function BubbleLoot:ListLoot(from, filter)
	if(from == nil) then
		for i,v in ipairs(BubbleLootList) do
			if((filter==nil) or (v[1]==filter)) then
				self:Print("<BL> " .. i .. " " .. v[1] .. " " .. v[2])
			end
		end
	else
		for i,v in ipairs(BubbleLootList) do
			if((filter==nil) or (v[1]==filter)) then
				SendChatMessage("<BL> " .. v[1] .. " " .. v[2], "WHISPER", nil, from)
			end
		end
	end
end

function BubbleLoot:ResetBids()
	tableBids={}
end

-- HANDLE_LOOT_ASSIGN
-- This function is invoked whenever someone loots something.  This will run even if you are not
-- the master looter (so it can be used to keep a quick record of loot recorded in an instance)
-- This function parses the loot assignment notification to update the loot table.
-- First it grabs out whomever received the loot, and then it checks the item to see whether it's
-- worth parsing or not.
function BubbleLoot:HandleLootAssign(msg)
	local name=string.match(msg, "(%a+)")
	if (name == "You") then
		name = UnitName("player")
	end 
	if(self:IsRollableItem(msg)) then
		local itemId = string.match(msg, "item:(%d+)")
		local itemName, itemLink, itemQual = GetItemInfo(itemId)
		self:Print("<BL> " .. name .. " " .. itemLink)
--		need to change this to identify what type of bid it is
		table.insert(BubbleLootList, {name, itemLink, itemId, "bid"})
		if(self.data.tableTally[name]) == nil then
			self.data.tableTally[name] = {1, 0, 0, 0}
		else
			self.data.tableTally[name][1] = self.data.tableTally[name][1]+1
		end
	end
end

-- UPDATE_LOOT_METHOD
-- This function
function BubbleLoot:UpdateLootMethod()
	local lootMethod, partyMaster, raidMaster=GetLootMethod()
	self.data.bInRaid=(partyMaster==nil)
	if(lootMethod=="master") then
		if(raidMaster == nil) then
			self.data.bAmML=(partyMaster==0)
		else
			self.data.bAmML=(UnitName("player")==GetRaidRosterInfo(raidMaster))
		end
	end
	if(self.data.bAmML) then
		self:Print("<BL> Am loot master")
	end
end